require "box2d"

application:setFps(30)
application:setBackgroundColor(0x000000)

world = b2.World.new(0, 9.8, true)
b2.setScale(100)

local function GIDEROS_Bitmap(fileName, left, top, filter)
	local texture = Texture.new(fileName, filter)	
	local bitmap = Bitmap.new(texture)	
	bitmap:setAnchorPoint(0.5, 0.5)	
	bitmap:setPosition(left + 0.5 * texture:getWidth(), top + 0.5 * texture:getHeight())
	stage:addChild(bitmap)
	return bitmap
end

local function GIDEROS_addBody(obj, typeBody, ...)
	local body = nil
	
	if typeBody == "static" then
		body = world:createBody{type = b2.STATIC_BODY}
	else
		body = world:createBody{type = b2.DYNAMIC_BODY}
	end
	
	body:setPosition(obj:getX(), obj:getY())
	
	for i=1, #arg do
		
		local properties = arg[i]
		local objShape = nil
		
		if properties.shape ~= nil then
			objShape = b2.PolygonShape.new()
			objShape:set(unpack(properties.shape))
		else
			if properties.radius ~= nil then
				objShape = b2.CircleShape.new(0, 0, properties.radius)			
			else		
				objShape = b2.PolygonShape.new()
				local ang = obj:getRotation()
				obj:setRotation(0)
				objShape:setAsBox(0.5 * obj:getWidth(), 0.5 * obj:getHeight())
				body:setAngle(ang * math.pi/180)
			end
		end
		
		local fixture = 
				{
					shape = objShape, 
					density = properties.density, 
					friction = properties.friction,
					restitution = properties.restitution
				}
		
		body:createFixture(fixture)
		
	end
	
	obj.body = body
end

local bar = GIDEROS_Bitmap("bar.png", 0, 0, true)
GIDEROS_addBody(bar, "static", {density = 1.0, friction = 0.3, restitution = 0.6, shape = {32.077,-14.869, -26.630,23.256, -32.077,14.869, 26.630,-23.256}})	
bar.body:setPosition(280, 280)

local knot1 = GIDEROS_Bitmap("knot.png", 0, 0, true)
GIDEROS_addBody(knot1, "static", {density = 1.0, friction = 0.8, restitution = 0.6})	
knot1.body:setPosition(13, 360)

local knot2 = GIDEROS_Bitmap("knot.png", 0, 0, true)
GIDEROS_addBody(knot2, "static", {density = 1.0, friction = 0.8, restitution = 0.6})	
knot2.body:setPosition(67, 360)

local ball = GIDEROS_Bitmap("ball.png", 0, 0, true)
GIDEROS_addBody(ball, "dynamic", {density = 1.0, friction = 0.3, restitution = 0.1, radius = 20})	
ball.body:setPosition(280, 55)


local function PerformPhysicsFrame(event)

    world:step(1/30, 8, 3)

    for i = 1, stage:getNumChildren() do
        local sprite = stage:getChildAt(i)
        if sprite.body then
            local body = sprite.body
            local bodyX, bodyY = body:getPosition()
            sprite:setPosition(bodyX, bodyY)
            sprite:setRotation(body:getAngle() * 180 / math.pi)
        end
    end	
end

stage:addEventListener(Event.ENTER_FRAME, PerformPhysicsFrame)

--local debugDraw = b2.DebugDraw.new()
--world:setDebugDraw(debugDraw)
--stage:addChild(debugDraw)
